from dataclasses import dataclass

@dataclass
class Reacts:
    like: str = "👍"
    unlike: str = "👎"
    heart: str = "❤"
    fire: str = "🔥"
    face_with_hearts: str = "🥰"
    bye: str = "👏"
    beaming_face: str = "😁"
    thinking: str = "🤔"
    exploding: str = "🤯"
    screaming: str = "😱"
    anger: str = "🤬"
    crying: str = "😢"
    party_popper: str = "🎉"
    star_struck: str = "🤩"
    vomiting: str = "🤮"
    shit: str = "💩"
    folded_hands: str = "🙏"
    ok_hand: str = "👌"
    dove: str = "🕊"
    lightning: str = "⚡"
    clown: str = "🤡"
    yawning: str = "🥱"
    woozy: str = "🥴"
    woow: str = "😍"
    whale: str = "🐳"
    heart_on_fire: str = "❤‍🔥"
    dark_moon: str = "🌚"
    hot_dog: str = "🌭"
    hundred: str = "💯"
    rolling_on_the_floor_laughing: str = "🤣"
    banana: str = "🍌"
    trophy: str = "🏆"
    broken_heart: str = "💔"
    raised_eyebrow: str = "🤨"
    neutral_face: str = "😐"
    strawberry: str = "🍓"
    champagne: str = "🍾"
    kiss: str = "💋"
    middle_finger: str = "🖕"
    smiling_devil: str = "😈"
    sleeping: str = "😴"
    crying_loudly: str = "😭"
    nerd_face: str = "🤓"
    ghost: str = "👻"
    technologist: str = "👨‍💻"
    eyes: str = "👀"
    jack_o_lantern: str = "🎃"
    see_no_evil: str = "🙈"
    angel: str = "😇"
    fearful_face: str = "😨"
    handshake: str = "🤝"
    writing_hand: str = "✍"
    hugging_face: str = "🤗"
    saluting_face: str = "🫡"
    santa_claus: str = "🎅"
    christmas_tree: str = "🎄"
    snowman: str = "☃"
    nail_polish: str = "💅"
    zany_face: str = "🤪"
    moai: str = "🗿"
    cool: str = "🆒"
    heart_with_arrow: str = "💘"
    hear_no_evil: str = "🙉"
    unicorn: str = "🦄"
    face_blowing_kiss: str = "😘"
    pill: str = "💊"
    speak_no_evil: str = "🙊"
    sunglasses: str = "😎"
    alien: str = "👾"
    man_shrugging: str = "🤷‍♂"
    woman_shrugging: str = "🤷‍♀"



    